;/********************************************************************/
;/********************************************************************/
;/******                                                        ******/
;/*                  S c a n n e r D r i v e r                       */
;/******                                                        ******/
;/*********************** NISCA Incorporated *************************/
;/********************************************************************/
;/*                                                                  */
;/*  PROJECT : NSK Scanner               PROJECT VERSION : 1.0       */
;/*  USAGE   : GEM Scan/NISCAN Driver    MODULE VERSION  : 1.0       */
;/*  OS      : MS\DOS                    COMPILER : MASM             */
;/*                                                                  */
;/*  PURPOSE: Production NISCAN Scanner driver for GEM Scan.         */
;/*                                                                  */
;/*                                                                  */
;/*  HISTORY:                                                        */
;/*    P.V.   M.V.      DATE     WHO   DESCRIPTION                   */
;/*    1.0    1.0     10/20/87   BW  Original creation               */
;/*    1.0    2.0     02/15/89   BW  Made NISCA Production version   */
;/*                                                                  */
;/********************************************************************/
;/*  (C)1989, NISCA Incorporated. This program is proprietary        */
;/*  information of NISCA Incorporated 1919 Old Denton Road,         */
;/*  Suite 104, Carrollton, TX 75006.  All rights reserved.  Coping  */
;/*  or Disclosure of this information are prohibited to             */
;/*  unauthorized persons.                                           */
;/********************************************************************/

;/********************************************************************/
;/*                                                                  */
;/*    EQUATES\CONSTANTS                                             */
;/*                                                                  */
;/********************************************************************/
        SC_STATUS       EQU     01H     ;Scanner status port
        SC_CNTRL        EQU     01H     ;Scanner control port
        SC_DATA         EQU     00H     ;Scanner data port
        SC_ON           EQU     02H     ;Scanner ON command port
        SC_OFF          EQU     03H     ;Scanner OFF command port
        TIME_OUT        EQU  0FFFFH     ;Scanner time-out for line     

;/********************************************************************/
;/*                                                                  */
;/*     DATA DECLARATIONS                                            */
;/*                                                                  */
;/********************************************************************/

PGROUP  GROUP   _TEXT
DGROUP  GROUP   _DATA
;**     input controls:
        EXTRN   _P_INDEX:WORD         ;* Port index from "c"
        EXTRN   _HS_B_SKIP:WORD       ;* Number of bytes to drop
        EXTRN   _HS_L_SKIP:WORD       ;* Number of lines to skip
        EXTRN   _HS_B_KEEP:WORD       ;* Number of bytes in line to keep
        EXTRN   _HS_L_KEEP:WORD       ;* Number of lines to keep
        EXTRN   _HS_B_SEG:WORD        ;* Buffer address
        EXTRN   _HS_B_OFS:WORD        ;*
        EXTRN   _HS_LEN:WORD          ;* Buffer length in paragraphs
        EXTRN   _HS_PORT:WORD         ;* Handy scanner port address
        EXTRN   _HS_DIT:WORD          ;* Dither control
        EXTRN   _HS_SDIT:WORD         ;* Software dither control
        EXTRN   _HS_INV:WORD          ;* Inverse video flag
        EXTRN   _HS_BRIGHT:WORD       ;* Brightness control
        EXTRN   _HS_RE:WORD           ;* Reduction control word

;**     output:
        EXTRN   _HS_CNT:DWORD          ;* length of image in bytes
        EXTRN   _HS_L:WORD            ;* number of scan lines
        EXTRN   _HS_W:WORD            ;* number of bytes wide

_DATA    SEGMENT PARA PUBLIC 'DATA'
        public  patch_str,ports,_port_num
patch_str       db      "zyxg"
_port_num       db      0
xfer_addr       db      0
short_name      db      "N I S C A N" ;* 13 character name with null term
                db      2 dup (0)
long_name       db      "NISCA Incorporated,  NISCAN Hand Scanner."
                db      52 dup (0)
port_choice     db      81 dup (0)
xfer_choice     db      81 dup (0)
com_file        db      82 dup (0)
ports           dw      0258h, 0278h, 0358h, 0378h
_DATA    ENDS

_TEXT   SEGMENT BYTE PUBLIC 'CODE'
        ASSUME  CS:PGROUP
        ASSUME  DS:DGROUP
;/********************************************************************/
;/*      INTERNAL DATA                                               */
;/********************************************************************/
        COPY_RIGHT      DB      '(C)1987 Comar Systek, Inc.'
        IMAGE_SEG       DW      0000H   ;Adjusted image buffer address
        END_SEG         DW      0000H   ;End address of image buffer
        SCAN_WIDTH      DW      0000H   ;Image buffer address
        W_KEEP          DW      0000H   ;Number of words to keep/line
        B_SKIP          DW      0000H   ;Number of bytes/line to skip
        INVRS           DW      0FFFFH  ;Invert the image.
        RAND            DW      80F2H   ;Random number

;********  Table of values for verticle line dither *********

V_D     DB 00001000b,00011100b,00011100b,00011100b,00011100b,00011100b,00011100b,00011100b
        DB 00001000b,00011100b,00011100b,00011100b,00011100b,00011100b,00011100b,00011100b
        DB 00011100b,00011100b,00011100b,00111110b,00011100b,00111110b,00011100b,00111110b
        DB 00011100b,00011100b,00011100b,00111110b,00011100b,00111110b,00011100b,00111110b
        DB 00011100b,00011100b,00011100b,00111110b,00011100b,00111110b,00011100b,00111110b
        DB 00011100b,00011100b,00011100b,00111110b,00011100b,00111110b,00011100b,00111110b
        DB 00011100b,00111110b,00111110b,00111110b,00111110b,00111110b,00111110b,01111111b
        DB 00011100b,00111110b,00111110b,00111110b,00111110b,00111110b,00111110b,01111111b
        DB 00011100b,00011110b,00011100b,00111110b,00011100b,00111110b,00111110b,00111110b
        DB 00011100b,00011110b,00011100b,00111110b,00011100b,00111110b,00111110b,00111110b
        DB 00011100b,00111110b,00111110b,00111110b,00111110b,00111110b,00111110b,01111111b
        DB 00011100b,00111110b,00111110b,00111110b,00111110b,00111110b,00111110b,01111111b
        DB 00011100b,00111110b,00111110b,00111110b,00111110b,00111110b,00111110b,01111111b
        DB 00011100b,00111110b,00111110b,00111110b,00111110b,00111110b,00111110b,01111111b
        DB 00111110b,00111110b,00111110b,01111111b,00111110b,01111111b,01111111b,01111111b
        DB 00111110b,00111110b,00111110b,01111111b,00111110b,01111111b,01111111b,01111111b
;
;******** Table of bit masks for Random Dot Defussion ************

DOT_M     DB 11000000b,01100000b,00110000b,00011000b,00001100b,00000110b,00000011b,10000001b
RND_M     DB 10000000b,01000000b,00100000b,00010000b,00001000b,00000100b,00000010b,00000001b


;/***********************************************************************
;**
;**     SET_PORT
;**
;**     Sets the port addresses according to the port number
;**
;**     input:
;**             none
;**
;**     output:
;**             none
;**
;***********************************************************************\
        PUBLIC  SET_PORT
SET_PORT PROC    NEAR
        MOV     BX,_P_INDEX             ;get the dynamic index
        TEST    BX,BX
        JZ      SP0001
        JMP     SP0002
SP0001: MOV     BL, _port_num            ;Load port number
        XOR     BH,BH
SP0002: SHL     BX, 1                   ;Calculate port address offset
        MOV     AX, ports[BX]           ;Load port address
        MOV     _HS_PORT, ax        ;Set control port
        RET
SET_PORT ENDP
;

;/***********************************************************************
;**
;**     _HS_ON
;**
;**     Interface to turn the NSK Handy Scanner ON 
;**
;**     input:
;**             none
;**
;**     output:
;**             none
;**
;***********************************************************************\
;
        PUBLIC  _HS_ON
_HS_ON   PROC    NEAR
        PUSH    AX              ;Save the REGS
        PUSH    BX
        PUSH    CX
        PUSH    DX
        PUSH    DI
        PUSH    SI
        PUSH    DS
        PUSH    ES
        PUSH    BP
        MOV     BP,SP
        MOV     AX,SEG _HS_B_OFS ;Set up the data seg 
        MOV     DS,AX
;**** Turn scanner ON
        CALL    SET_PORT
        MOV     DX,_HS_PORT      ;Get the scanner port address
        ADD     DX,SC_ON        ;Point to the turn on port
        IN      AL,DX           ;Turn the scanner on
;****   
        POP     BP
        POP     ES              ;Recover the REG before we exit
        POP     DS
        POP     SI
        POP     DI
        POP     DX
        POP     CX
        POP     BX
        POP     AX
        RET
_HS_ON   ENDP
;/***********************************************************************
;**
;**     _HS_OFF
;**
;**     Interface to turn the NSK Handy Scanner OFF 
;**
;**     input:
;**             none
;**
;**     output:
;**             none
;**
;***********************************************************************\
;
        PUBLIC  _HS_OFF
_HS_OFF  PROC    NEAR
        PUSH    AX              ;Save the REGS
        PUSH    BX
        PUSH    CX
        PUSH    DX
        PUSH    DI
        PUSH    SI
        PUSH    DS
        PUSH    ES
        PUSH    BP
        MOV     BP,SP
        MOV     AX,SEG _HS_B_OFS ;Set up the data seg 
        MOV     DS,AX
;**** Turn scanner OFF
        CALL    SET_PORT
        MOV     DX,_HS_PORT      ;Get the scanner port address
        ADD     DX,SC_OFF       ;Point to the turn OFF port
        IN      AL,DX           ;Turn the scanner off
;****   
        POP     BP
        POP     ES              ;Recover the REG before we exit
        POP     DS
        POP     SI
        POP     DI
        POP     DX
        POP     CX
        POP     BX
        POP     AX
        RET

_HS_OFF  ENDP
;************************************************************************
;*  DO_BIT
;*
;*  Subroutine to ship a bit to the scanner
;*  AL - bit 1 is the data bit
;*  DX - scanner control port address
;*
;************************************************************************
DO_BIT PROC    NEAR
        PUSH    CX
        AND     AL,01H          ;Mask the rest of the byte
        XOR     AL,0FFH         ;Flip the bits
        OUT     DX,AL           ;Clock is off
        MOV     CX,5000         ;Must hold for long enough to charge line
        LOOP    $               ;Wait for a while
        AND     AL,0FDH         ;Turn on clock bit
        OUT     DX,AL           ;Send data and clock
        MOV     CX,5000         ;Hold data and clock
        LOOP    $
        POP     CX
        RET                     ;return
DO_BIT  ENDP
;/***********************************************************************
;**
;**     _HS_SCAN
;**
;**     Interface to scan image into the supplied buffer
;**
;**     input:
;**             _HS_B_SKIP - Number of bytes to drop
;**             _HS_L_SKIP - Number of lines to skip
;*              _HS_B_KEEP - Number of bytes in line to keep
;**             _HS_L_KEEP - Number of lines to keep
;**             _HS_B_SEG  - Buffer segment address
;**             _HS_B_OFS  - Buffer offset
;**             _HS_LEN    - Buffer length
;**             _HS_PORT   - Handy scanner port address
;**     output:
;**             _HS_CNT - length of image in bytes
;**             _HS_L   - number of scan lines
;**             _HS_W   - number of bytes wide
;**             _HS_RE  - Handy Scanner Reduction Flag
;**
;***********************************************************************\
;
        PUBLIC  _HS_SCAN
_HS_SCAN PROC    NEAR
        PUSH    AX              ;Save the REGS
        PUSH    BX
        PUSH    CX
        PUSH    DX
        PUSH    DI
        PUSH    SI
        PUSH    DS
        PUSH    ES
        PUSH    BP
        MOV     BP,SP
        MOV     AX,SEG _HS_B_OFS ;Set up the data seg 
        MOV     DS,AX
        MOV     AX,_HS_B_OFS    ;Set up buffer to begin on seg boundry
        MOV     BX,_HS_B_SEG    ;
        CMP     AX,AX           ;is it alread OK
        JZ      BUF_OK          ;YES - Buffer is ok
        ADD     AX,0FH          ;(AX + 15) / 16
        SHR     AX,1   
        SHR     AX,1   
        SHR     AX,1   
        SHR     AX,1   
        ADD     BX,AX           ;BX = SEG + Adjusted Offset
BUF_OK: ADD     BX,1            ;Add 16 to give adjustment space
        MOV     IMAGE_SEG,BX    ;Save the fixed segmet reg
        ADD     BX,_HS_LEN      ;add the buffer length
        SUB     BX,07H          ;Subtract one scan line 
        MOV     END_SEG,BX      ;Save the end seg address of the buffer
;************************************************************************
;*  Loop to read in image
;*      AH = 00H - Byte not ready
;*      AH = 01H - Data byte ready
;*      AH = 80H - New line
;*
;*      AH - Status
;*      AL - Data byte
;*      DS - Data Segment
;*      DX - Data Segment (Temp)
;*      CX - Time out count
;*      BX - Char pointer
;*
;*************************************************************************
;**** Turn scanner ON
        CALL    SET_PORT
        MOV     DX,_HS_PORT      ;Get the scanner port address
        ADD     DX,SC_ON        ;Point to the turn on port
        IN      AL,DX           ;Turn the scanner on
        MOV     CX,TIME_OUT     ;Get my timeout value
        LOOP    $               ; wait for scanner power to stabilize
        LOOP    $               ; Minimum of 300msec
        LOOP    $               ; 
        LOOP    $               ; 
        MOV     AX,_HS_INV      ;Get the inverse mask
        MOV     INVRS,AX        ;Set the inverse mask
;**** Program scanner
        MOV     DX,_HS_PORT
        ADD     DX,SC_CNTRL     ;Point to the control port
        MOV     AL,0FFH         ;The bits must be sent out bit 7 first
        OUT     DX,AL           ;Clear all bits
;****** Set brightness level  
        MOV     CL,3            ; through away the first 3 bits
        MOV     BX,_HS_BRIGHT   ; Get the Brightness control word
        SHL     BL,CL
        MOV     CX,5            ; Five bits to go
S_BRIGHT: MOV     AL,0          ; Get a zero bit
        SHL     BL,1            ; Shift to the next bit
        JNC     S_B1            ; Not a 1 send the 0
        MOV     AL,01           ; It's a 1 send a 1
S_B1:   CALL    DO_BIT          ; Send the bit
        LOOP    S_BRIGHT        ; Go do the next bit              
;****** Set hardware dither control
        MOV     BX,_HS_DIT      ;get the dither control word
        XOR     BX,0FFH         ;flip the dither control bits
        MOV     AL,BL           ;get the second bit first
        SHR     AL,1
        CALL    DO_BIT          ;send the bit
        MOV     AL,BL           ;
        CALL    DO_BIT          ;send bit 1 of dither control
;****** Set reduction/resolution bit
        MOV     AX,_HS_RE       ;get the reduction bit
        CALL    DO_BIT          ;Out put the first bit
;**** Read and discard the first _HS_SKIP scan lines
        MOV     DX,_HS_PORT      ;Get the scanner port address
        MOV     SI,0            ;Clear line count in SI
W_0001: IN      AX,DX           ;Wait for a new line
        TEST    AH,10000000b    ;is this the first line
        JNS     W_0001          ;Not yet
        TEST    AH,00010000b    ;Is this a reduced image
        JZ      W_0002          ; No full width
        MOV     AX,04H          ;Half width/16
        MOV     SCAN_WIDTH,AX
        JMP     S_0001
W_0002: MOV     AX,07H           ;Full width/16
        MOV     SCAN_WIDTH,AX
S_0001: MOV     BX,_HS_L_SKIP      ;Number of lines to discard
S_0002: MOV     CX,TIME_OUT     ;Set the timeout count                   4
S_0003: IN      AX,DX           ;Read Status/Data byte                  12
        TEST    AH,10000000b    ;Test the status                         4
        LOOPZ   S_0002          ;Loop waiting for char or time out      18/6
        DEC     BX              ;Next line
        JNZ     S_0002
;**** Start keeping the image
        PUSH    DS              ;Save My data segment
        MOV     AX,IMAGE_SEG    ;Get the image segment                  13
        CLI
NEXT0:  MOV     DS,AX           ;Point segment at Image buffer           2
        XOR     BX,BX           ;                                        3
NEXT1:  MOV     CX,TIME_OUT     ;Set the timeout count                   4
NEXT2:  IN      AX,DX           ;Read Status/Data
        TEST    AH,11000000b    ;Test the status                         4
        LOOPZ   NEXT2           ;Loop waiting for char or time out      18/6
        JS      NEWLIN          ;Next line
        JZ      RTN_IMG         ;Finished go return the image
GOTBYT: MOV     DS:[BX],AL      ;Save the byte
        INC     BX              ;Increment pointer
        JMP     NEXT1           ;Go do next byte
NEWLIN: MOV     AX,DS           ;Get the current segment
        INC     SI              ;Count the lines
        ADD     AX,SCAN_WIDTH   ;Add line length
        CMP     AX,END_SEG      ;Is this all
        JAE     RTN_IMG         ; YES - Go return image
        JMP     NEXT0           ;Go do next line
;***************************************************************************
RTN_IMG EQU     $                ;
        STI
;**** Turn the scanner off
        ADD     DX,SC_CNTRL     ; point to control port
        MOV     AL,0
        OUT     DX,AL            ;Clear control port
        ADD     DX,2             ;Read the OFF port
        IN      AX,DX            ;to turn OFF the Scanner
;**** Move the image into the correct positon in the image buffer
        POP     DS              ;Reset the data segment
        PUSH    DS              ;Save the data segemnt
MOVE_I: MOV     AX,_HS_B_SEG    ;Get the start of the buffer SEG
        MOV     ES,AX
        MOV     AX,_HS_B_OFS    ;Get the buffer start OFFSET
        MOV     DI,AX
        MOV     AX,SI           ;Get the number of lines scanned
        CMP     AX,_HS_L_KEEP   ;Return the lessor(SI,KEEP)
        JAE     MOV_LS          ;Number scanned is greater than _HS_L_KEEP
        MOV     _HS_L_KEEP,AX   ;We did not scan that many lines
        CMP     AX,0            ;
        JNZ     MOV_LS
        POP     DS              ;Get the extra DS of the stack
        JMP     GET_OUT         ;Zero lines, lets quit
MOV_LS: CLD                     ;lets increment 
        MOV     BX,_HS_L_KEEP   ;Get the number of to keep
        MOV     AX,_HS_B_SKIP   ;Get the number of bytes to skip
        MOV     B_SKIP,AX       ;Save the number of bytes per line to skip
        MOV     AX,_HS_B_KEEP   ;
        MOV     W_KEEP,AX       ;Save the number of words per line
        MOV     AX,IMAGE_SEG    ;Get the start address of the image
        MOV     DS,AX
MOV_L:  MOV     AX,B_SKIP
        MOV     SI,AX
        MOV     CX,W_KEEP       ;get the number of words to keep
MOV_W:  LODSW
        XCHG    AL,AH
        XOR     AX,INVRS        ;invert the bits
        STOSW
        LOOP    MOV_W
        DEC     BX              ;Is this all of the lines to keep?
        JZ      POSTDITH        ;  YES -  check for post dithering
        MOV     AX,DI           ;Increment buffer seg for next row
        SHR     AX,1            ;  Offset/16
        SHR     AX,1            ;  Offset/16
        SHR     AX,1            ;  Offset/16
        SHR     AX,1            ;  Offset/16
        MOV     CX,ES           ;  
        ADD     AX,CX           ;  Buffer segment updated
        MOV     ES,AX
        MOV     AX,DI           ;Update the buffer OFFSET
        AND     AX,000FH        ;  Keep Lower 4 bits OFFSET MOD 16
        MOV     DI,AX 
        MOV     AX,DS           ;Increment Scan seg reg to next row
        ADD     AX,SCAN_WIDTH   ;
        MOV     DS,AX           ;
        JMP     MOV_L           ;Do the next line
;*******************************************************************
POSTDITH:POP    DS              ;Recover the data segment
        MOV     AX,_HS_SDIT      ;Get the soft dither control
        CMP     AX,04H          ;Is this a software dither
        JL      GETOUT         ;  NO just return the image
        JG      DOTSWAP         ;  Check Random DotSwap Deffusion #4
;********************************************************************
VERTCL: MOV     AX,_HS_B_OFS    ;Get the buffer start OFFSET
        MOV     DI,AX
        CLD                     ;lets increment 
VER_LS: MOV     DX,_HS_L_KEEP   ;Get the number of to keep
        MOV     AX,_HS_B_SEG    ;Get the start of the buffer SEG
        MOV     DS,AX
VER_L:  MOV     CX,W_KEEP       ;get the number of words to keep
        SAL     CX,1            ;make it bytes
VER_B:  MOV     BL,[DI]         ; get the byte
        AND     BX,007FH        ; make it an index into the table
        MOV     AL,V_D[BX]      ; get the new value from table
        MOV     [DI],AL         ; save the new byte
        INC     DI              ; count the character
        LOOP    VER_B
        DEC     DX              ;Is this all of the lines to keep?
        JZ      GETOUT         ;  YES -  it's finished
        MOV     AX,DI           ;Increment buffer seg for next row
        SHR     AX,1            ;  Offset/16
        SHR     AX,1            ;  Offset/16
        SHR     AX,1            ;  Offset/16
        SHR     AX,1            ;  Offset/16
        MOV     CX,DS           ;  
        ADD     AX,CX           ;  Buffer segment updated
        MOV     DS,AX
        MOV     AX,DI           ;Update the buffer OFFSET
        AND     AX,000FH        ;  Keep Lower 4 bits OFFSET MOD 16
        MOV     DI,AX 
        JMP     VER_L           ;Do the next line
;***********************************************************
GETOUT: JMP     GET_OUT         ;
;***********************************************************
DOTSWAP: CMP    AX,05H          ;Is it Dot Swap Defussion #4
        JG      RANDDOT         ; NO - Go check Random dot defussion
DOT_DL: MOV     AX,_HS_B_OFS    ;Get the buffer start OFFSET
        MOV     DI,AX
        CLD                     ;lets increment 
DOT_LS: MOV     DX,_HS_L_KEEP   ;Get the number of to keep
        MOV     AX,_HS_B_SEG    ;Get the start of the buffer SEG
        MOV     DS,AX
DOT_L:  MOV     CX,W_KEEP       ;get the number of words to keep
        SAL     CX,1            ;make it bytes
DOT_B:  MOV     AL,[DI]         ;Get the byte
        MOV     BX,RAND         ;Get current random number
        SAL     BX,1            ; shift left 1
        JNC     DOT_001         ; Skip XOR
        XOR     BX,8002H        ; insert some bits
DOT_001: MOV     RAND,BX         ; Save my number
        AND     BX,0007H        ; make it an index into the table
        MOV     AL,DOT_M[BX]      ; get the new mask from table
        AND     AL,[DI]         ;  Check for BIT transition
        JZ      DOT_NXT         ;   NO - check the next one
        CMP     AL,DOT_M[BX]      ;
        JE      DOT_NXT         ;   NO - try the next one
        MOV     AL,DOT_M[BX]      ;   YES - so swap the bits
        XOR     [DI],AL
DOT_NXT: INC     DI              ; count the character
        LOOP    DOT_B
        DEC     DX              ;Is this all of the lines to keep?
        JZ      GETOUT         ;  YES -  it's finished
        MOV     AX,DI           ;Increment buffer seg for next row
        SHR     AX,1            ;  Offset/16
        SHR     AX,1            ;  Offset/16
        SHR     AX,1            ;  Offset/16
        SHR     AX,1            ;  Offset/16
        MOV     CX,DS           ;  
        ADD     AX,CX           ;  Buffer segment updated
        MOV     DS,AX
        MOV     AX,DI           ;Update the buffer OFFSET
        AND     AX,000FH        ;  Keep Lower 4 bits OFFSET MOD 16
        MOV     DI,AX 
        JMP     DOT_L           ;Do the next line
;*************************************************************
RANDDOT: CMP    AX,06H          ;Is it Random Dot Defussion #5
        JG      GETOUT         ; NO - Go check for pseudo-gray
RANDL:  MOV     AX,_HS_B_OFS    ;Get the buffer start OFFSET
        MOV     DI,AX
        CLD                     ;lets increment 
RND_LS: MOV     DX,_HS_L_KEEP   ;Get the number of to keep
        MOV     AX,_HS_B_SEG    ;Get the start of the buffer SEG
        MOV     DS,AX
RND_L:  MOV     CX,W_KEEP       ;get the number of words to keep
        SAL     CX,1            ;make it bytes
RND_B:  MOV     AL,[DI]         ;Get the byte
        AND     AL,00FH         ;Check for bit runs
        CMP     AL,0            ;Is this a white
        JE      RND_NXT         ; YES -Skip it
        CMP     AL,00FH         ;Is this a black
        JE      RND_NXT         ; YES - I'm an EOP 
        MOV     AL,[DI]         ;Get the byte
        AND     AL,0F0H         ;Check for bit runs
        CMP     AL,0            ;Is this a white
        JE      RND_NXT         ; YES -Skip it
        CMP     AL,0F0H         ;Is this a black
        JE      RND_NXT         ; YES - I'm an EOP 
        MOV     AL,[DI]         ;Get the byte
        AND     AL,03CH         ;Check for bit runs
        CMP     AL,0            ;Is this a white
        JE      RND_NXT         ; YES -Skip it
        CMP     AL,03CH         ;Is this a black
        JE      RND_NXT         ; YES - I'm an EOP 
        MOV     BX,RAND         ;Get current random number
        SAL     BX,1            ; shift left 1
        JNC     RND_001         ; Skip XOR
        XOR     BX,8002H        ; insert some bits
RND_001: MOV     RAND,BX        ; Save my number
        AND     BX,0007H        ; make it an index into the table
        MOV     AL,RND_M[BX]    ; get the new value from table
        XOR     [DI],AL         ; Make the new byte
RND_002: INC    BX              ; get next mask
        AND     BX,0007H        ; make it an index into the table
        MOV     AL,DOT_M[BX]    ; get the new value from table
        XOR     AL,0FFH
        AND     [DI],AL         ; Make the new byte
RND_NXT: INC     DI             ; count the character
        LOOP    RND_B
        DEC     DX              ;Is this all of the lines to keep?
        JZ      GET_OUT         ;  YES -  it's finished
        MOV     AX,DI           ;Increment buffer seg for next row
        SHR     AX,1            ;  Offset/16
        SHR     AX,1            ;  Offset/16
        SHR     AX,1            ;  Offset/16
        SHR     AX,1            ;  Offset/16
        MOV     CX,DS           ;  
        ADD     AX,CX           ;  Buffer segment updated
        MOV     DS,AX
        MOV     AX,DI           ;Update the buffer OFFSET
        AND     AX,000FH        ;  Keep Lower 4 bits OFFSET MOD 16
        MOV     DI,AX 
        JMP     RND_L           ;Do the next line
;******************************************************************** 
GET_OUT: POP     BP
        POP     ES              ;Recover the REG before we exit
        POP     DS
        POP     SI
        POP     DI
        POP     DX
        POP     CX
        POP     BX
        POP     AX
        RET
_HS_SCAN ENDP


_TEXT   ENDS

        end
