/********************************************************************/
/********************************************************************/
/******                                                        ******/
/*                      N I S C A N . C                             */
/******                                                        ******/
/*********************** COMAR SYSTEK, INC. *************************/
/********************************************************************/
/*                                                                  */
/*  PROJECT : NISCAN                    PROJECT VERSION : 1.0       */
/*  USAGE   : NISCAN Driver Test Bed    MODULE VERSION  : 1.2       */
/*  OS      : MS-DOS 3.31               COMPILER : MSC 4.0          */
/*                                                                  */
/*  PURPOSE: This is a test program to test NISCAN Driver's.        */
/*                                                                  */
/*                                                                  */
/*  HISTORY:                                                        */
/*    P.V.   M.V.      DATE     WHO   DESCRIPTION                   */
/*           1.0      10/18/88  RLW   Original creation             */
/*                                                                  */
/********************************************************************/
/*  (C)1989 NISCA Incorporated This program is proprietary          */
/*  information of NISCA Incorporated, 1919 Old Denton Road,        */
/*  Suite 104, Carrollton TX 75006.  All rights reserved.  Copying  */
/*  or Disclosure of this information is prohibited to              */
/*  unauthorized persons.                                           */
/********************************************************************/

/********************************************************************/
/*                                                                  */
/*    INCLUDE FILES                                                 */
/*                                                                  */
/********************************************************************/
#include    "portab.h"
#include    "machine.h"
#include    "dos.h"
#include    "utilxtrn.h"
#include    "global.h"
#include    "hsdata.h"

#include    <STDIO.H>
#include    <CONIO.H>


EXTERN   VOID     HS_ON();
EXTERN   VOID     HS_SCAN();
EXTERN   VOID     HS_OFF();
EXTERN   VOID     DISPLAY_IMAGE();
EXTERN   VOID     ALLOC_BUF();

/********************************************************************/
/*                                                                  */
/*    DEFINES/TYPE DEFINITIONS                                      */
/*                                                                  */
/********************************************************************/
/********************************************************************/
/*                                                                  */
/*     GLOBAL DATA DECLARATIONS                                     */
/*                                                                  */
/********************************************************************/

int   resolution;
int   intensity;
char  invert;
char  mode;
char  dither;
char  sdither;
int   active;
int   scan_port;
int   i;
int   max_lines;    
int   max_length;
/********************************************************************/
/*                                                                  */
/*     PROCEDURES/FUNCTIONS                                         */
/*                                                                  */
/********************************************************************/

/*****************************************************************************
**
**  scan()
**
**  Scan image
**
**  input:
**      buff            Long pointer to image buffer
**      length          length of image buffer
**
*****************************************************************************/
    VOID
scan()
{
   hs_bright = intensity;
   hs_inv = 0;
   if ((invert == 'y') || (invert=='Y')) hs_inv = -1;
   switch (dither){
      case 'N':
      case 'n':
         hs_dit = 0;
         break;
      case 'S':
      case 's':
         hs_dit = 1;
         break;
      case 'B':
      case 'b':
         hs_dit = 2;
         break;
      case 'M':
      case 'm':
         hs_dit = 3;
         break;
   }
   switch (sdither){
      case 'N':
      case 'n':
         hs_sdit= 0;
         break;
      case 'L':
      case 'l':
         hs_sdit= 4;
         break;
      case 'P':
      case 'p':
         hs_sdit = 5;
         break;
      case 'R':
      case 'r':
         hs_sdit = 6;
         break;
   }
   if ((mode=='T')||(mode=='t')) hs_sdit=hs_dit=0;
   
   hs_re    = (resolution > 100)?0:1;


   hs_b_skip = 0;                      /*  Number of bytes to drop  */
   hs_l_skip = 2;                      /*  Number of lines to skip  */
   hs_b_keep = (hs_re)?27:53;          /*  Number of bytes/2 in line to keep  */
   hs_l_keep = max_lines;              /*  Number of lines to keep  */


   HS_SCAN();                   /* Do the scan in HSASM.ASM */
   hs_cnt = ((LONG)hs_l_keep) * ((LONG)hs_b_keep) * 2;
}
/********************************************************************/
/*     MAIN                                                         */
/********************************************************************/
main()
{
   char  key[80];
   char  *result;
   
   resolution  = 200;
   intensity   = 0;
   invert      = 'N';
   mode        = 'T';
   dither      = 'B';
   sdither     = 'N';
   active      = 1;  /* Set the active flag to true */
   scan_port   = 0x258;

   printf("\nNISCAN Test Program        Version 1.0    10/18/88 \n\n" );
   printf("\ncopyright (c) NISCA Incorporated 1988, 1989        \n\n" );

   ALLOC_BUF();   /* allocate an image buffer as large as possible */

   max_lines = hs_len / 7;
   max_length = max_lines / 200;

   while (active){
      printf("\nNISCAN>");
      if (key[0] = getch()){
         switch (key[0]){
            case 'S':
            case 's': printf("\n Starting scan...");
               scan();
               display_image();
               break;
            case 'M':
            case 'm':
               printf("\n (T)ext Mode");
               printf("\n (H)alftone");
               printf("\n Set Mode (%c):",mode);
               if(gets(key)) sscanf(key,"%c",&mode);
               break;
            case 'D':
            case 'd':
               printf("\n (N)one");
               printf("\n (B)ayer");
               printf("\n (S)piral");
               printf("\n (M)esh");
               printf("\n Set Halftone Dither (%c):",dither);
               if(gets(key)) sscanf(key, "%c",&dither);
               break;
            case 'L':
            case 'l':
               printf("\n (N)one");
               printf("\n (L)ine");
               printf("\n (P)ixal Swap ");
               printf("\n (R)andom Dot Diffusion ");
               printf("\n Set Halftone filter (%c):",sdither);
               if(gets(key)) sscanf(key, "%c", &sdither);
               break;
            case 'B':
            case 'b': printf("\n Set brightness [-16 to 15] (%d):",intensity);
               if(gets(key)) sscanf(key," %d",&intensity);
               if (intensity > 15) intensity = 15;
               if (intensity < -16) intensity = -16;
               break;
            case 'R':
            case 'r': printf("\n Set resolution 100 or 200dpi (%d):",resolution);
               if(gets(key)) sscanf(key," %d",&resolution);
               if (resolution > 100){
                  resolution = 200;
                  max_lines = hs_len / 7;
               }
               else {
                  resolution = 100;
                  max_lines = hs_len / 4;
               }
               max_length = max_lines / resolution;
               break;
            case 'I':
            case 'i': printf("\n Set Image Invertion (%c):",invert);
               if(gets(key)) sscanf(key,"%c",&invert);
               break;
            case 'V':
            case 'v': printf("\n Displaying current image...\n");
               DISPLAY_IMAGE();
               break;
            case 'O':
            case 'o': printf("\n Turn scanner ON");
               HS_ON();
               break;
            case 'P':
            case 'p':
               printf("\n Scanner port address (%xh,%d):",scan_port,p_index);
               if (gets( key )) sscanf(key, " %x", &scan_port);
               switch (scan_port){
                  case 0x258:
                     p_index = 0;
                     break;
                  case 0x278:
                     p_index = 1;
                     break;
                  case 0x358:
                     p_index = 2;
                     break;
                  case 0x378:
                     p_index = 3;
                     break;
                }
                break;
            case 'F':
            case 'f': printf("\n Turn scanner OFF");
               HS_OFF();
               break;
            case 'H':
            case 'h':
               printf("\n ============ HELP =========== \n");
               printf(" B - set the scanner Brightness level \n");
               printf(" M - set the scan Mode (Text or Halftone) \n");
               printf(" D - set the hardware halftone dither \n");
               printf(" L - set the software halftone filter \n"); 
               printf(" I - set the image Invertion flag \n");
               printf(" R - set the scanner resolution \n");
               printf(" S - scan/display an image \n");
               printf(" V - display the current image \n");
               printf(" ------------------------------ \n");
               printf(" P - set scanner PORT Address \n");
               printf(" O - turn scanner ON \n");
               printf(" F - turn scanner OFF \n");
               printf(" ------------------------------ \n");
               printf(" H - display this screen \n");
               printf(" Q - quit \n");
               printf(" ? - display current scanner settings \n\n");
               break;
            case 'Q':
            case 'q': printf("\n  say by...");
               active = 0;
               break;
            case '?':
            case '/':
               printf("\n Current scanner parameters:\n");
               printf("       Port address        %xh\n",scan_port);
               printf("       Mode                %c (Text or Halftone)\n",mode);
               printf("       Halftone Dither     %c\n",dither);
               printf("       Halftone Filter     %c\n",sdither);
               printf("       Resolution          %d dpi\n",resolution);
               printf("       Invert Image        %c\n",invert);
               printf("       Brightness          %d\n",intensity);
               printf("       Max Scan length     %d inches\n",max_length);
               break;
          }
      }
   }

}

